@@ -33,6 +33,7 @@
 #include "objlava.h"
 #include "objsnow.h"
 #include "objswmp.h"
+#include "race.h"
 #include "icn.h"
 
 namespace ICN
@@ -43,7 +44,7 @@ namespace ICN
         const char* string;
     };
 
-    static const icnmap_t icnmap[] = {
+    const icnmap_t icnmap[] = {
 	{ ADVBORDE,	"ADVBORDE.ICN" },
 	{ ADVBORD,	"ADVBORD.ICN" },
 	{ ADVBTNS,	"ADVBTNS.ICN" },
@@ -915,15 +916,23 @@ namespace ICN
 	{ TREASURE,	"MANUAL.ICN" },
 	{ YELLOW_FONT,	"MANUAL.ICN" },
 	{ YELLOW_SMALFONT,"MANUAL.ICN" },
+	{ BATTLESKIP,   "MANUAL.ICN" },
+	{ BATTLEWAIT,   "MANUAL.ICN" },
+	{ BATTLEAUTO,   "MANUAL.ICN" },
+	{ BATTLESETS,   "MANUAL.ICN" },
+	{ BTNCONFIG,    "MANUAL.ICN" },
+	{ BTNBATTLEONLY,"MANUAL.ICN" },
+	{ BOAT12,       "MANUAL.ICN" },
+	{ BTNGIFT,      "MANUAL.ICN" },
 
-	{ UNKNOWN,	"UNKNOWN.ICN" },
+	{ UNKNOWN,	"UNKNOWN" },
     };
 
     u8 missile9(float, float);
     u8 missile7(float, float);
 }
 
-const char* ICN::GetString(const icn_t icn)
+const char* ICN::GetString(const icn_t & icn)
 {
     return icnmap[icn].string;
 }
@@ -942,7 +951,7 @@ void ICN::Header::Load(const u8* p)
     offset_data = ReadLE32(&p[9]);
 }
 
-u16 ICN::AnimationFrame(const icn_t icn, const u16 start, const u32 ticket, const u8 quantity)
+u16 ICN::AnimationFrame(const icn_t & icn, const u8 & start, const u32 & ticket, bool quantity)
 {
     switch(icn)
     {
@@ -1528,23 +1537,6 @@ u16 ICN::AnimationFrame(const icn_t icn, const u16 start, const u32 ticket, cons
     return 0;
 }
 
-bool ICN::RequiresAlpha(const ICN::icn_t icn)
-{
-    switch(icn)
-    {
-        case ICN::SYSTEM:
-        case ICN::SYSTEME:
-        case ICN::BUYBUILD:
-        case ICN::BUYBUILE:
-        case ICN::BOOK:
-            return false;
-        default:
-            break;
-    }
-    
-    return true;
-}
-
 ICN::icn_t ICN::PORTxxxx(u8 id)
 {
     switch(id)
@@ -1630,12 +1622,18 @@ ICN::icn_t ICN::PORTxxxx(u8 id)
     return ICN::UNKNOWN;
 }
 
-bool ICN::NeedMinify4PocketPC(icn_t icn, u16 index)
+bool ICN::NeedMinify4PocketPC(const icn_t & icn, const u16 & index)
 {
+    if(Settings::Get().ExtBattleSoftWait() &&
+	icn == TEXTBAR && (index == 0 || index == 1)) return false;
+
     switch(icn)
     {
 	case TOWNWIND:
 	    return index == 0 || index == 1;
+
+	case HISCORE:
+	    return index == 6;
 	//
 	case PUZZLE:
 	case HSBKG:
@@ -1701,6 +1699,7 @@ bool ICN::NeedMinify4PocketPC(icn_t icn, u16 index)
         case DISRRAY:
 	case MORALEG:
 	case MORALEB:
+	case EXPMRL:
 
 	// castle
         case CASTBKGB:
@@ -1776,6 +1775,11 @@ bool ICN::NeedMinify4PocketPC(icn_t icn, u16 index)
 	case COBJ0030:
 	case COBJ0031:
 
+	case BATTLESKIP:
+	case BATTLEWAIT:
+	case BATTLEAUTO:
+	case BATTLESETS:
+
 	    return true;
 
 	case TEXTBAR:
@@ -1790,28 +1794,6 @@ bool ICN::NeedMinify4PocketPC(icn_t icn, u16 index)
     return false;
 }
 
-bool ICN::SkipBottomForRedrawHeroes(icn_t icn, u16 index)
-{
-    switch(icn)
-    {
-	case ICN::OBJNTWBA:
-        case ICN::ROAD:
-        case ICN::STREAM:
-            return true;
-
-	case ICN::OBJNCRCK: return ObjWasteLand::isPassable(icn, index);
-	case ICN::OBJNDIRT: return ObjDirt::isPassable(icn, index);
-	case ICN::OBJNDSRT: return ObjDesert::isPassable(icn, index);
-	case ICN::OBJNGRA2:
-	case ICN::OBJNGRAS: return ObjGrass::isPassable(icn, index);
-	case ICN::OBJNLAVA: return ObjLava::isPassable(icn, index);
-	case ICN::OBJNSNOW: return ObjSnow::isPassable(icn, index);
-	case ICN::OBJNSWMP: return ObjSwamp::isPassable(icn, index);
-        default: break;
-    }
-    return false;
-}
-
 u8 ICN::missile9(float dx, float dy)
 {
     if(0 == dx)		return dy > 0 ? 0 : 8;
@@ -1845,7 +1827,7 @@ u8 ICN::missile7(float dx, float dy)
     return dy > 0 ? 1 : 5;
 }
 
-u8 ICN::GetMissIndex(icn_t icn, s16 dx, s16 dy)
+u8 ICN::GetMissIndex(const icn_t & icn, const s16 & dx, const s16 & dy)
 {
     switch(icn)
     {
@@ -1866,7 +1848,7 @@ u8 ICN::GetMissIndex(icn_t icn, s16 dx, s16 dy)
     return 0;
 }
 
-bool ICN::isModifiedSprite(const ICN::icn_t icn)
+bool ICN::isModifiedSprite(const ICN::icn_t & icn)
 {
     switch(icn)
     {
@@ -1878,6 +1860,15 @@ bool ICN::isModifiedSprite(const ICN::icn_t icn)
         case TELEPORT3:
         case FOUNTAIN:
         case TREASURE:
+	case BATTLESKIP:
+	case BATTLEWAIT:
+	case BATTLESETS:
+	case BATTLEAUTO:
+	case BUYMAX:
+	case BTNCONFIG:
+	case BTNGIFT:
+	case BTNBATTLEONLY:
+	case BOAT12:
 	    return true;
 
 	default: break;
@@ -1985,7 +1976,7 @@ ICN::icn_t ICN::FromString(const char* str)
     return ptr->type;
 }
 
-bool ICN::HighlyObjectSprite(ICN::icn_t icn, u16 index)
+bool ICN::HighlyObjectSprite(const ICN::icn_t & icn, const u8 & index)
 {
     switch(icn)
     {
@@ -2048,3 +2039,51 @@ bool ICN::HighlyObjectSprite(ICN::icn_t icn, u16 index)
     }
     return false;
 }
+
+ICN::icn_t ICN::Get4Captain(u8 race)
+{
+    switch(race)
+    {
+        case Race::BARB: return CSTLCAPB;
+        case Race::KNGT: return CSTLCAPK;
+        case Race::NECR: return CSTLCAPN;
+        case Race::SORC: return CSTLCAPS;
+        case Race::WRLK: return CSTLCAPW;
+        case Race::WZRD: return CSTLCAPZ;
+        default: break;
+    }
+
+    return UNKNOWN;
+}
+
+ICN::icn_t ICN::Get4Building(u8 race)
+{
+    switch(race)
+    {
+        case Race::BARB: return CSTLBARB;
+        case Race::KNGT: return CSTLKNGT;
+        case Race::NECR: return CSTLNECR;
+        case Race::SORC: return CSTLSORC;
+        case Race::WRLK: return CSTLWRLK;
+        case Race::WZRD: return CSTLWZRD;
+        default: break;
+    }
+
+    return UNKNOWN;
+}
+
+ICN::icn_t ICN::Get4Castle(u8 race)
+{
+    switch(race)
+    {
+        case Race::BARB: return CASTLEB;
+        case Race::KNGT: return CASTLEK;
+        case Race::NECR: return CASTLEN;
+        case Race::SORC: return CASTLES;
+        case Race::WRLK: return CASTLEW;
+        case Race::WZRD: return CASTLEZ;
+        default: break;
+    }
+
+    return UNKNOWN;
+}
