@@ -32,26 +32,27 @@
 #include "splitter.h"
 #include "marketplace.h"
 #include "dialog.h"
-#include "localclient.h"
 
-void RedrawFromResource(const Point & pt, const Resource::funds_t & rs);
+void RedrawFromResource(const Point & pt, const Funds & rs);
 void RedrawToResource(const Point & pt, bool showcost, bool tradingPost, u8 from_resource = 0);
-void GetStringTradeCosts(std::string & str, u8 rs_from, u8 rs_to, bool tradingPost);
+std::string GetStringTradeCosts(u8 rs_from, u8 rs_to, bool tradingPost);
 u16 GetTradeCosts(u8 rs_from, u8 rs_to, bool tradingPost);
 
 class TradeWindowGUI
 {
 public:
-    TradeWindowGUI(const Rect & rt) : 
+    TradeWindowGUI(const Rect & rt) :
 	pos_rt(rt), back(rt.x - 5, rt.y + 15, rt.w + 10, 160),
-	tradpost(Settings::Get().EvilInterface() ? ICN::TRADPOSE : ICN::TRADPOST)
+	tradpost(Settings::Get().ExtGameEvilInterface() ? ICN::TRADPOSE : ICN::TRADPOST)
     {
 	back.Save();
 
+        buttonGift.SetPos(pos_rt.x + (pos_rt.w - AGG::GetICN(tradpost, 17).w()) / 2, pos_rt.y + 120);
         buttonTrade.SetPos(pos_rt.x + (pos_rt.w - AGG::GetICN(tradpost, 17).w()) / 2, pos_rt.y + 150);
 	buttonLeft.SetPos(pos_rt.x + 11, pos_rt.y + 129);
 	buttonRight.SetPos(pos_rt.x + 220, pos_rt.y + 129);
 
+	buttonGift.SetSprite(ICN::BTNGIFT, 0, 1);
 	buttonTrade.SetSprite(tradpost, 15, 16);
 	buttonLeft.SetSprite(tradpost, 3, 4);
 	buttonRight.SetSprite(tradpost, 5, 6);
@@ -70,9 +71,11 @@ public:
     void RedrawInfoBuySell(u32 count_sell, u32 count_buy);
     void ShowTradeArea(u8 resourceFrom, u8 resourceTo, u32 max_buy, u32 max_sell, u32 count_buy, u32 count_sell, bool fromTradingPost);
 
+    Rect   buttonMax;
     Button buttonTrade;
     Button buttonLeft;
     Button buttonRight;
+    Button buttonGift;
     Splitter splitter;
 
 private:
@@ -92,12 +95,16 @@ void TradeWindowGUI::ShowTradeArea(u8 resourceFrom, u8 resourceTo, u32 max_buy,
     if(resourceFrom == resourceTo || (Resource::GOLD != resourceTo && 0 == max_buy))
     {
         cursor.Hide();
+	splitter.Hide();
         back.Restore();
         Rect dst_rt(pos_rt.x, pos_rt.y + 30, pos_rt.w, 100);
         TextBox(_("You have received quite a bargain. I expect to make no profit on the deal. Can I interest you in any of my other wares?"), Font::BIG, dst_rt);
+        buttonGift.SetDisable(false);
 	buttonTrade.SetDisable(true);
         buttonLeft.SetDisable(true);
         buttonRight.SetDisable(true);
+        buttonGift.Draw();
+	buttonMax = Rect();
         cursor.Show();
         display.Flip();
     }
@@ -110,47 +117,47 @@ void TradeWindowGUI::ShowTradeArea(u8 resourceFrom, u8 resourceTo, u32 max_buy,
         const Sprite & bar = AGG::GetICN(tradpost, 1);
         dst_pt.x = pos_rt.x + (pos_rt.w - bar.w()) / 2 - 2;
         dst_pt.y = pos_rt.y + 128;
-        display.Blit(bar, dst_pt);
+        bar.Blit(dst_pt);
         splitter.SetRange(0, (Resource::GOLD == resourceTo ? max_sell : max_buy));
-        Resource::resource_t rs_from = static_cast<Resource::resource_t>(resourceFrom);
-        Resource::resource_t rs_to   = static_cast<Resource::resource_t>(resourceTo);
         u16 exchange_rate = GetTradeCosts(resourceFrom, resourceTo, fromTradingPost);
 	std::string message;
         if(Resource::GOLD == resourceTo)
         {
             message = _("I can offer you %{count} for 1 unit of %{resfrom}.");
             String::Replace(message, "%{count}", exchange_rate);
-            String::Replace(message, "%{resfrom}", Resource::String(rs_from));
+            String::Replace(message, "%{resfrom}", Resource::String(resourceFrom));
         }
         else
         {
             message = _("I can offer you 1 unit of %{resto} for %{count} units of %{resfrom}.");
-            String::Replace(message, "%{resto}", Resource::String(rs_to));
-            String::Replace(message, "%{resfrom}", Resource::String(rs_from));
+            String::Replace(message, "%{resto}", Resource::String(resourceTo));
+            String::Replace(message, "%{resfrom}", Resource::String(resourceFrom));
             String::Replace(message, "%{count}", exchange_rate);
         }
         TextBox(message, Font::BIG, Rect(pos_rt.x, pos_rt.y + 30, pos_rt.w, 100));
-        const Sprite & sprite_from = AGG::GetICN(ICN::RESOURCE, Resource::GetIndexSprite2(rs_from));
+        const Sprite & sprite_from = AGG::GetICN(ICN::RESOURCE, Resource::GetIndexSprite2(resourceFrom));
         dst_pt.x = pos_rt.x + pos_rt.w / 2 - 70 - sprite_from.w() / 2;
         dst_pt.y = pos_rt.y + 115 - sprite_from.h();
-        display.Blit(sprite_from, dst_pt);
-        message.clear();
-        String::AddInt(message, count_sell);
-        const Sprite & sprite_to = AGG::GetICN(ICN::RESOURCE, Resource::GetIndexSprite2(rs_to));
+        sprite_from.Blit(dst_pt);
+        const Sprite & sprite_to = AGG::GetICN(ICN::RESOURCE, Resource::GetIndexSprite2(resourceTo));
         dst_pt.x = pos_rt.x + pos_rt.w / 2 + 70 - sprite_to.w() / 2;
         dst_pt.y = pos_rt.y + 115 - sprite_to.h();
-        display.Blit(sprite_to, dst_pt);
-        message.clear();
-        String::AddInt(message, count_buy);
+        sprite_to.Blit(dst_pt);
         const Sprite & sprite_fromto = AGG::GetICN(tradpost, 0);
         dst_pt.x = pos_rt.x + pos_rt.w / 2 - sprite_fromto.w() / 2;
         dst_pt.y = pos_rt.y + 90;
-        display.Blit(sprite_fromto, dst_pt);
-        Text text(_("Qty to trade"), Font::SMALL);
+        sprite_fromto.Blit(dst_pt);
+	buttonMax = Rect(dst_pt.x, dst_pt.y, sprite_fromto.w(), sprite_fromto.h());
+        Text text("max", Font::SMALL);
+        dst_pt.x = buttonMax.x + (buttonMax.w - text.w()) / 2;
+        dst_pt.y = buttonMax.y + 2;
+        text.Blit(dst_pt);
+	text.Set(_("Qty to trade"), Font::SMALL);
         dst_pt.x = pos_rt.x + (pos_rt.w - text.w()) / 2;
         dst_pt.y = pos_rt.y + 110;
         text.Blit(dst_pt);
 
+        buttonGift.SetDisable(true);
         buttonTrade.SetDisable(false);
         buttonLeft.SetDisable(false);
         buttonRight.SetDisable(false);
@@ -160,6 +167,7 @@ void TradeWindowGUI::ShowTradeArea(u8 resourceFrom, u8 resourceTo, u32 max_buy,
         buttonRight.Draw();
 
         RedrawInfoBuySell(count_sell, count_buy);
+	splitter.Show();
         cursor.Show();
         display.Flip();
     }
@@ -168,20 +176,16 @@ void TradeWindowGUI::ShowTradeArea(u8 resourceFrom, u8 resourceTo, u32 max_buy,
 void TradeWindowGUI::RedrawInfoBuySell(u32 count_sell, u32 count_buy)
 {
     Point dst_pt;
-    std::string message;
 
-    String::AddInt(message, count_sell);
     textSell.Hide();
-    textSell.SetText(message);
+    textSell.SetText(GetString(count_sell));
     dst_pt.x = pos_rt.x + pos_rt.w / 2 - 70 - textSell.w() / 2;
     dst_pt.y = pos_rt.y + 116;
     textSell.SetPos(dst_pt);
     textSell.Show();
 
-    message.clear();
-    String::AddInt(message, count_buy);
     textBuy.Hide();
-    textBuy.SetText(message);
+    textBuy.SetText(GetString(count_buy));
     dst_pt.x = pos_rt.x + pos_rt.w / 2 + 70 - textBuy.w() / 2;
     dst_pt.y = pos_rt.y + 116;
     textBuy.SetPos(dst_pt);
@@ -191,7 +195,7 @@ void TradeWindowGUI::RedrawInfoBuySell(u32 count_sell, u32 count_buy)
 void Dialog::Marketplace(bool fromTradingPost)
 {
     Display & display = Display::Get();
-    const ICN::icn_t tradpost = Settings::Get().EvilInterface() ? ICN::TRADPOSE : ICN::TRADPOST;
+    const ICN::icn_t tradpost = Settings::Get().ExtGameEvilInterface() ? ICN::TRADPOSE : ICN::TRADPOST;
     const std::string & header = _("Marketplace");
 
     Cursor & cursor = Cursor::Get();
@@ -213,22 +217,23 @@ void Dialog::Marketplace(bool fromTradingPost)
 
     TradeWindowGUI gui(pos_rt);
 
-    Kingdom & kingdom = world.GetMyKingdom();
+    Kingdom & kingdom = world.GetKingdom(Settings::Get().CurrentColor());
     const Sprite & spritecursor = AGG::GetICN(tradpost, 14);
 
     const std::string & header_from = _("Your Resources");
 
-    Resource::funds_t fundsFrom = kingdom.GetFundsResource();
+    Funds fundsFrom = kingdom.GetFunds();
     u8 resourceFrom = 0;
     const Point pt1(pos_rt.x, pos_rt.y + 190);
-    std::vector<Rect> rectsFrom(7);
-    rectsFrom[0] = Rect(pt1.x, pt1.y, 34, 34);		// wood
-    rectsFrom[1] = Rect(pt1.x + 37, pt1.y, 34, 34);	// mercury
-    rectsFrom[2] = Rect(pt1.x + 74, pt1.y, 34, 34);	// ore
-    rectsFrom[3] = Rect(pt1.x, pt1.y + 37, 34, 34);	// sulfur
-    rectsFrom[4] = Rect(pt1.x + 37, pt1.y + 37, 34, 34);// crystal
-    rectsFrom[5] = Rect(pt1.x + 74, pt1.y + 37, 34, 34);// gems
-    rectsFrom[6] = Rect(pt1.x + 37, pt1.y + 74, 34, 34);// gold
+    Rects rectsFrom;
+    rectsFrom.reserve(7);
+    rectsFrom.push_back(Rect(pt1.x, pt1.y, 34, 34));		// wood
+    rectsFrom.push_back(Rect(pt1.x + 37, pt1.y, 34, 34));	// mercury
+    rectsFrom.push_back(Rect(pt1.x + 74, pt1.y, 34, 34));	// ore
+    rectsFrom.push_back(Rect(pt1.x, pt1.y + 37, 34, 34));	// sulfur
+    rectsFrom.push_back(Rect(pt1.x + 37, pt1.y + 37, 34, 34));	// crystal
+    rectsFrom.push_back(Rect(pt1.x + 74, pt1.y + 37, 34, 34));	// gems
+    rectsFrom.push_back(Rect(pt1.x + 37, pt1.y + 74, 34, 34));	// gold
     SpriteCursor cursorFrom(spritecursor);
     text.Set(header_from, Font::SMALL);
     dst_pt.x = pt1.x + (108 - text.w()) / 2;
@@ -238,17 +243,18 @@ void Dialog::Marketplace(bool fromTradingPost)
 
     const std::string & header_to = _("Available Trades");
 
-    Resource::funds_t fundsTo;
+    Funds fundsTo;
     u8 resourceTo = 0;
     const Point pt2(138 + pos_rt.x, pos_rt.y + 190);
-    std::vector<Rect> rectsTo(7);
-    rectsTo[0] = Rect(pt2.x, pt2.y, 34, 34);		// wood
-    rectsTo[1] = Rect(pt2.x + 37, pt2.y, 34, 34);	// mercury
-    rectsTo[2] = Rect(pt2.x + 74, pt2.y, 34, 34);	// ore
-    rectsTo[3] = Rect(pt2.x, pt2.y + 37, 34, 34);	// sulfur
-    rectsTo[4] = Rect(pt2.x + 37, pt2.y + 37, 34, 34);	// crystal
-    rectsTo[5] = Rect(pt2.x + 74, pt2.y + 37, 34, 34);	// gems
-    rectsTo[6] = Rect(pt2.x + 37, pt2.y + 74, 34, 34);	// gold
+    Rects rectsTo;
+    rectsTo.reserve(7);
+    rectsTo.push_back(Rect(pt2.x, pt2.y, 34, 34));		// wood
+    rectsTo.push_back(Rect(pt2.x + 37, pt2.y, 34, 34));		// mercury
+    rectsTo.push_back(Rect(pt2.x + 74, pt2.y, 34, 34));		// ore
+    rectsTo.push_back(Rect(pt2.x, pt2.y + 37, 34, 34));		// sulfur
+    rectsTo.push_back(Rect(pt2.x + 37, pt2.y + 37, 34, 34));	// crystal
+    rectsTo.push_back(Rect(pt2.x + 74, pt2.y + 37, 34, 34));	// gems
+    rectsTo.push_back(Rect(pt2.x + 37, pt2.y + 74, 34, 34));	// gold
     SpriteCursor cursorTo(spritecursor);
     text.Set(header_to, Font::SMALL);
     dst_pt.x = pt2.x + (108 - text.w()) / 2;
@@ -258,10 +264,12 @@ void Dialog::Marketplace(bool fromTradingPost)
 
     u32 count_sell = 0;
     u32 count_buy = 0;
-    
+
     u32 max_sell = 0;
     u32 max_buy = 0;
 
+    Rect & buttonMax = gui.buttonMax;
+    Button & buttonGift = gui.buttonGift;
     Button & buttonTrade = gui.buttonTrade;
     Button & buttonLeft = gui.buttonLeft;
     Button & buttonRight = gui.buttonRight;
@@ -273,120 +281,92 @@ void Dialog::Marketplace(bool fromTradingPost)
     dst_pt.y = pos_rt.y + pos_rt.h - sprite_exit.h();
     Button buttonExit(dst_pt, tradpost, 17, 18);
 
+    buttonGift.Draw();
     buttonExit.Draw();
-
     cursor.Show();
     display.Flip();
 
     LocalEvent & le = LocalEvent::Get();
-   
+
     // message loop
     while(le.HandleEvents())
     {
+        if(buttonGift.isEnable()) le.MousePressLeft(buttonGift) ? buttonGift.PressDraw() : buttonGift.ReleaseDraw();
         if(buttonTrade.isEnable()) le.MousePressLeft(buttonTrade) ? buttonTrade.PressDraw() : buttonTrade.ReleaseDraw();
         if(buttonLeft.isEnable()) le.MousePressLeft(buttonLeft) ? buttonLeft.PressDraw() : buttonLeft.ReleaseDraw();
         if(buttonRight.isEnable()) le.MousePressLeft(buttonRight) ? buttonRight.PressDraw() : buttonRight.ReleaseDraw();
 
         le.MousePressLeft(buttonExit) ? buttonExit.PressDraw() : buttonExit.ReleaseDraw();
 
-        if(le.MouseClickLeft(buttonExit) || le.KeyPress(KEY_RETURN) || le.KeyPress(KEY_ESCAPE)) break;
-	
+        if(le.MouseClickLeft(buttonExit) || HotKeyCloseWindow) break;
+
+	if(buttonGift.isEnable() &&
+	    le.MouseClickLeft(buttonGift))
+	{
+            cursor.Hide();
+	    Dialog::MakeGiftResource();
+	    fundsFrom = kingdom.GetFunds();
+	    RedrawFromResource(pt1, fundsFrom);
+            cursor.Show();
+            display.Flip();
+	}
+
         // click from
         for(u8 ii = 0; ii < rectsFrom.size(); ++ii)
         {
             const Rect & rect_from = rectsFrom[ii];
-            
+
             if(le.MouseClickLeft(rect_from))
             {
-                switch(ii)
-                {
-                    case 0:
-                        resourceFrom = Resource::WOOD;
-                        max_sell = fundsFrom.wood;
-                        break;
-                    case 1:
-                        resourceFrom = Resource::MERCURY;
-                        max_sell = fundsFrom.mercury;
-                        break;
-                    case 2:
-                        resourceFrom = Resource::ORE;
-                        max_sell = fundsFrom.ore;
-                        break;
-                    case 3:
-                        resourceFrom = Resource::SULFUR;
-                        max_sell = fundsFrom.sulfur;
-                        break;
-                    case 4:
-                        resourceFrom = Resource::CRYSTAL;
-                        max_sell = fundsFrom.crystal;
-                        break;
-                    case 5:
-                        resourceFrom = Resource::GEMS;
-                        max_sell = fundsFrom.gems;
-                        break;
-                    case 6:
-                        resourceFrom = Resource::GOLD;
-                        max_sell = fundsFrom.gold;
-                        break;
-                    default: break;
-                }
-                
+		resourceFrom = Resource::FromIndexSprite2(ii);
+                max_sell = fundsFrom.Get(resourceFrom);
+
                 if(GetTradeCosts(resourceFrom, resourceTo, fromTradingPost))
                 {
-                    max_buy = Resource::GOLD == resourceTo ? 
+                    max_buy = Resource::GOLD == resourceTo ?
                         max_sell * GetTradeCosts(resourceFrom, resourceTo, fromTradingPost) :
                         max_sell / GetTradeCosts(resourceFrom, resourceTo, fromTradingPost);
                 }
-                
+
                 count_sell = 0;
                 count_buy = 0;
-                
+
                 cursor.Hide();
                 cursorFrom.Move(rect_from.x - 2, rect_from.y - 2);
                 cursorFrom.Show();
-                
+
                 if(resourceTo) cursorTo.Hide();
                 RedrawToResource(pt2, true, fromTradingPost, resourceFrom);
                 if(resourceTo) cursorTo.Show();
                 if(resourceTo) gui.ShowTradeArea(resourceFrom, resourceTo, max_buy, max_sell, count_buy, count_sell, fromTradingPost);
-                
+
                 cursor.Show();
                 display.Flip();
             }
         }
-        
+
         // click to
         for(u8 ii = 0; ii < rectsTo.size(); ++ii)
         {
             const Rect & rect_to = rectsTo[ii];
-            
+
             if(le.MouseClickLeft(rect_to))
             {
-                switch(ii)
-                {
-                    case 0: resourceTo = Resource::WOOD; break;
-                    case 1: resourceTo = Resource::MERCURY; break;
-                    case 2: resourceTo = Resource::ORE; break;
-                    case 3: resourceTo = Resource::SULFUR; break;
-                    case 4: resourceTo = Resource::CRYSTAL; break;
-                    case 5: resourceTo = Resource::GEMS; break;
-                    case 6: resourceTo = Resource::GOLD; break;
-                    default: break;
-                }
-                
+		resourceTo = Resource::FromIndexSprite2(ii);
+
                 if(GetTradeCosts(resourceFrom, resourceTo, fromTradingPost))
                 {
-                    max_buy = Resource::GOLD == resourceTo ? 
+                    max_buy = Resource::GOLD == resourceTo ?
                         max_sell * GetTradeCosts(resourceFrom, resourceTo, fromTradingPost) :
                         max_sell / GetTradeCosts(resourceFrom, resourceTo, fromTradingPost);
                 }
-                
+
                 count_sell = 0;
                 count_buy = 0;
-                
+
                 cursor.Hide();
                 cursorTo.Move(rect_to.x - 2, rect_to.y - 2);
-                
+
                 if(resourceFrom)
                 {
                     cursorTo.Hide();
@@ -398,287 +378,183 @@ void Dialog::Marketplace(bool fromTradingPost)
                 display.Flip();
             }
         }
-        
+
         // move splitter
         if(buttonLeft.isEnable() && buttonRight.isEnable() && max_buy && le.MousePressLeft(splitter.GetRect()))
         {
             u32 seek = (le.GetMouseCursor().x - splitter.GetRect().x) * 100 / splitter.GetStep();
-            
+
             if(seek < splitter.Min()) seek = splitter.Min();
             else
             if(seek > splitter.Max()) seek = splitter.Max();
-            
+
             count_buy = seek * (Resource::GOLD == resourceTo ? GetTradeCosts(resourceFrom, resourceTo, fromTradingPost) : 1);
             count_sell = seek * (Resource::GOLD == resourceTo ? 1: GetTradeCosts(resourceFrom, resourceTo, fromTradingPost));
-            
+
             cursor.Hide();
             splitter.Move(seek);
             gui.RedrawInfoBuySell(count_sell, count_buy);
             cursor.Show();
             display.Flip();
         }
-        
+        else
+	// click max
+	if(buttonMax.w && max_buy && le.MouseClickLeft(buttonMax))
+	{
+	    const u32 & max = splitter.Max();
+
+            count_buy  = max * (Resource::GOLD == resourceTo ? GetTradeCosts(resourceFrom, resourceTo, fromTradingPost) : 1);
+            count_sell = max * (Resource::GOLD == resourceTo ? 1: GetTradeCosts(resourceFrom, resourceTo, fromTradingPost));
+
+            cursor.Hide();
+            splitter.Move(max);
+            gui.RedrawInfoBuySell(count_sell, count_buy);
+            cursor.Show();
+            display.Flip();
+	}
+
         // trade
         if(buttonTrade.isEnable() && le.MouseClickLeft(buttonTrade) && count_sell && count_buy)
         {
-            kingdom.OddFundsResource(Resource::funds_t(static_cast<Resource::resource_t>(resourceFrom), count_sell));
-            kingdom.AddFundsResource(Resource::funds_t(static_cast<Resource::resource_t>(resourceTo), count_buy));
-#ifdef WITH_NET
-	    FH2LocalClient::SendMarketSellResource(kingdom, resourceFrom, count_sell, fromTradingPost);
-	    FH2LocalClient::SendMarketBuyResource(kingdom, resourceTo, count_buy, fromTradingPost);
-#endif            
+            kingdom.OddFundsResource(Funds(resourceFrom, count_sell));
+            kingdom.AddFundsResource(Funds(resourceTo, count_buy));
+
             resourceTo = resourceFrom = Resource::UNKNOWN;
             gui.ShowTradeArea(resourceFrom, resourceTo, 0, 0, 0, 0, fromTradingPost);
 
-            fundsFrom = kingdom.GetFundsResource();
+            fundsFrom = kingdom.GetFunds();
             cursorTo.Hide();
             cursorFrom.Hide();
             RedrawFromResource(pt1, fundsFrom);
             RedrawToResource(pt2, false, fromTradingPost, resourceFrom);
             display.Flip();
         }
-        
+
         // decrease trade resource
         if(count_buy &&
            ((buttonLeft.isEnable() && le.MouseClickLeft(gui.buttonLeft)) ||
             le.MouseWheelDn(splitter.GetRect())))
         {
             count_buy -= Resource::GOLD == resourceTo ? GetTradeCosts(resourceFrom, resourceTo, fromTradingPost) : 1;
-            
+
             count_sell -= Resource::GOLD == resourceTo ? 1: GetTradeCosts(resourceFrom, resourceTo, fromTradingPost);
-            
+
             cursor.Hide();
             splitter.Backward();
             gui.RedrawInfoBuySell(count_sell, count_buy);
             cursor.Show();
             display.Flip();
         }
-        
+
         // increase trade resource
         if( count_buy < max_buy &&
             ((buttonRight.isEnable() && le.MouseClickLeft(buttonRight)) ||
              le.MouseWheelUp(splitter.GetRect())))
         {
             count_buy += Resource::GOLD == resourceTo ? GetTradeCosts(resourceFrom, resourceTo, fromTradingPost) : 1;
-            
+
             count_sell += Resource::GOLD == resourceTo ? 1: GetTradeCosts(resourceFrom, resourceTo, fromTradingPost);
-            
+
             cursor.Hide();
             splitter.Forward();
             gui.RedrawInfoBuySell(count_sell, count_buy);
             cursor.Show();
-            display.Flip();            
+            display.Flip();
         }
     }
 }
 
-void RedrawFromResource(const Point & pt, const Resource::funds_t & rs)
+void RedrawResourceSprite(const Surface & sf, s16 px, s16 py, s32 value)
 {
     Display & display = Display::Get();
-    const ICN::icn_t tradpost = Settings::Get().EvilInterface() ? ICN::TRADPOSE : ICN::TRADPOST;
-    std::string str;
-    Point dst_pt;
     Text text;
+    Point dst_pt(px, py);
+
+    sf.Blit(dst_pt, display);
+    text.Set(GetString(value), Font::SMALL);
+    dst_pt.x += (34 - text.w()) / 2;
+    dst_pt.y += 21;
+    text.Blit(dst_pt);
+}
+
+void RedrawFromResource(const Point & pt, const Funds & rs)
+{
+    const ICN::icn_t tradpost = Settings::Get().ExtGameEvilInterface() ? ICN::TRADPOSE : ICN::TRADPOST;
 
     // wood
-    dst_pt.x = pt.x;
-    dst_pt.y = pt.y;
-    display.Blit(AGG::GetICN(tradpost, 7), dst_pt);
-    str.clear();
-    String::AddInt(str, rs.wood);
-    text.Set(str, Font::SMALL);
-    dst_pt.x += (34 - text.w()) / 2;
-    dst_pt.y += 21;
-    text.Blit(dst_pt);
-    
+    RedrawResourceSprite(AGG::GetICN(tradpost, 7), pt.x, pt.y, rs.wood);
     // mercury
-    dst_pt.x = pt.x + 37;
-    dst_pt.y = pt.y;
-    display.Blit(AGG::GetICN(tradpost, 8), dst_pt);
-    str.clear();
-    String::AddInt(str, rs.mercury);
-    text.Set(str);
-    dst_pt.x += (34 - text.w()) / 2;
-    dst_pt.y += 21;
-    text.Blit(dst_pt);
-
+    RedrawResourceSprite(AGG::GetICN(tradpost, 8), pt.x + 37, pt.y, rs.mercury);
     // ore
-    dst_pt.x = pt.x + 74;
-    dst_pt.y = pt.y;
-    display.Blit(AGG::GetICN(tradpost, 9), dst_pt);
-    str.clear();
-    String::AddInt(str, rs.ore);
-    text.Set(str);
-    dst_pt.x += (34 - text.w()) / 2;
-    dst_pt.y += 21;
-    text.Blit(dst_pt);
-
+    RedrawResourceSprite(AGG::GetICN(tradpost, 9), pt.x + 74, pt.y, rs.ore);
     // sulfur
-    dst_pt.x = pt.x;
-    dst_pt.y = pt.y + 37;
-    display.Blit(AGG::GetICN(tradpost, 10), dst_pt);
-    str.clear();
-    String::AddInt(str, rs.sulfur);
-    text.Set(str);
-    dst_pt.x += (34 - text.w()) / 2;
-    dst_pt.y += 21;
-    text.Blit(dst_pt);
-
+    RedrawResourceSprite(AGG::GetICN(tradpost, 10), pt.x, pt.y + 37, rs.sulfur);
     // crystal
-    dst_pt.x = pt.x + 37;
-    dst_pt.y = pt.y + 37;
-    display.Blit(AGG::GetICN(tradpost, 11), dst_pt);
-    str.clear();
-    String::AddInt(str, rs.crystal);
-    text.Set(str);
-    dst_pt.x += (34 - text.w()) / 2;
-    dst_pt.y += 21;
-    text.Blit(dst_pt);
-
+    RedrawResourceSprite(AGG::GetICN(tradpost, 11), pt.x + 37, pt.y + 37, rs.crystal);
     // gems
-    dst_pt.x = pt.x + 74;
-    dst_pt.y = pt.y + 37;
-    display.Blit(AGG::GetICN(tradpost, 12), dst_pt);
-    str.clear();
-    String::AddInt(str, rs.gems);
-    text.Set(str);
-    dst_pt.x += (34 - text.w()) / 2;
-    dst_pt.y += 21;
-    text.Blit(dst_pt);
-
+    RedrawResourceSprite(AGG::GetICN(tradpost, 12), pt.x + 74, pt.y + 37, rs.gems);
     // gold
-    dst_pt.x = pt.x + 37;
-    dst_pt.y = pt.y + 74;
-    display.Blit(AGG::GetICN(tradpost, 13), dst_pt);
-    str.clear();
-    String::AddInt(str, rs.gold);
-    text.Set(str);
-    dst_pt.x += (34 - text.w()) / 2;
-    dst_pt.y += 21;
-    text.Blit(dst_pt);
+    RedrawResourceSprite(AGG::GetICN(tradpost, 13), pt.x + 37, pt.y + 74, rs.gold);
+}
+
+void RedrawResourceSprite2(const Surface & sf, s16 px, s16 py, bool show, u8 from, u8 res, bool trading)
+{
+    Display & display = Display::Get();
+    Point dst_pt(px, py);
+
+    sf.Blit(dst_pt, display);
+
+    if(show)
+    {
+	Text text(GetStringTradeCosts(from, res, trading), Font::SMALL);
+	dst_pt.x += (34 - text.w()) / 2;
+	dst_pt.y += 21;
+	text.Blit(dst_pt);
+    }
 }
 
 void RedrawToResource(const Point & pt, bool showcost, bool tradingPost, u8 from_resource)
 {
-    Display & display = Display::Get();
-    const ICN::icn_t tradpost = Settings::Get().EvilInterface() ? ICN::TRADPOSE : ICN::TRADPOST;
-    std::string str;
-    Point dst_pt;
-    Text text;
-    text.Set(Font::SMALL);
+    const ICN::icn_t tradpost = Settings::Get().ExtGameEvilInterface() ? ICN::TRADPOSE : ICN::TRADPOST;
 
     // wood
-    dst_pt.x = pt.x;
-    dst_pt.y = pt.y;
-    display.Blit(AGG::GetICN(tradpost, 7), dst_pt);
-    if(showcost)
-    {
-	GetStringTradeCosts(str, from_resource, Resource::WOOD, tradingPost);
-	text.Set(str);
-	dst_pt.x += (34 - text.w()) / 2;
-	dst_pt.y += 21;
-	text.Blit(dst_pt);
-    }
-    
+    RedrawResourceSprite2(AGG::GetICN(tradpost, 7), pt.x, pt.y, showcost, from_resource, Resource::WOOD, tradingPost);
     // mercury
-    dst_pt.x = pt.x + 37;
-    dst_pt.y = pt.y;
-    display.Blit(AGG::GetICN(tradpost, 8), dst_pt);
-    if(showcost)
-    {
-	GetStringTradeCosts(str, from_resource, Resource::MERCURY, tradingPost);
-	text.Set(str);
-	dst_pt.x += (34 - text.w()) / 2;
-	dst_pt.y += 21;
-	text.Blit(dst_pt);
-    }
-
+    RedrawResourceSprite2(AGG::GetICN(tradpost, 8), pt.x + 37, pt.y, showcost, from_resource, Resource::MERCURY, tradingPost);
     // ore
-    dst_pt.x = pt.x + 74;
-    dst_pt.y = pt.y;
-    display.Blit(AGG::GetICN(tradpost, 9), dst_pt);
-    if(showcost)
-    {
-	GetStringTradeCosts(str, from_resource, Resource::ORE, tradingPost);
-	text.Set(str);
-	dst_pt.x += (34 - text.w()) / 2;
-	dst_pt.y += 21;
-	text.Blit(dst_pt);
-    }
-
+    RedrawResourceSprite2(AGG::GetICN(tradpost, 9), pt.x + 74, pt.y, showcost, from_resource, Resource::ORE, tradingPost);
     // sulfur
-    dst_pt.x = pt.x;
-    dst_pt.y = pt.y + 37;
-    display.Blit(AGG::GetICN(tradpost, 10), dst_pt);
-    if(showcost)
-    {
-	GetStringTradeCosts(str, from_resource, Resource::SULFUR, tradingPost);
-	text.Set(str);
-	dst_pt.x += (34 - text.w()) / 2;
-	dst_pt.y += 21;
-	text.Blit(dst_pt);
-    }
-
+    RedrawResourceSprite2(AGG::GetICN(tradpost, 10), pt.x, pt.y + 37, showcost, from_resource, Resource::SULFUR, tradingPost);
     // crystal
-    dst_pt.x = pt.x + 37;
-    dst_pt.y = pt.y + 37;
-    display.Blit(AGG::GetICN(tradpost, 11), dst_pt);
-    if(showcost)
-    {
-	GetStringTradeCosts(str, from_resource, Resource::CRYSTAL, tradingPost);
-	text.Set(str);
-	dst_pt.x += (34 - text.w()) / 2;
-	dst_pt.y += 21;
-	text.Blit(dst_pt);
-    }
-
+    RedrawResourceSprite2(AGG::GetICN(tradpost, 11), pt.x + 37, pt.y + 37, showcost, from_resource, Resource::CRYSTAL, tradingPost);
     // gems
-    dst_pt.x = pt.x + 74;
-    dst_pt.y = pt.y + 37;
-    display.Blit(AGG::GetICN(tradpost, 12), dst_pt);
-    if(showcost)
-    {
-	GetStringTradeCosts(str, from_resource, Resource::GEMS, tradingPost);
-	text.Set(str);
-	dst_pt.x += (34 - text.w()) / 2;
-	dst_pt.y += 21;
-	text.Blit(dst_pt);
-    }
-
+    RedrawResourceSprite2(AGG::GetICN(tradpost, 12), pt.x + 74, pt.y + 37, showcost, from_resource, Resource::GEMS, tradingPost);
     // gold
-    dst_pt.x = pt.x + 37;
-    dst_pt.y = pt.y + 74;
-    display.Blit(AGG::GetICN(tradpost, 13), dst_pt);
-    if(showcost)
-    {
-	GetStringTradeCosts(str, from_resource, Resource::GOLD, tradingPost);
-	text.Set(str);
-	dst_pt.x += (34 - text.w()) / 2;
-	dst_pt.y += 21;
-	text.Blit(dst_pt);
-    }
+    RedrawResourceSprite2(AGG::GetICN(tradpost, 13), pt.x + 37, pt.y + 74, showcost, from_resource, Resource::GOLD, tradingPost);
 }
 
-void GetStringTradeCosts(std::string & str, u8 rs_from, u8 rs_to, bool tradingPost)
+std::string GetStringTradeCosts(u8 rs_from, u8 rs_to, bool tradingPost)
 {
-    if(str.size()) str.clear();
+    std::string res;
 
     if(rs_from == rs_to)
     {
-	str = _("n/a");
-	
-	return;
+	res = _("n/a");
+    }
+    else
+    {
+	if(Resource::GOLD != rs_from && Resource::GOLD != rs_to)
+	    res = "1/";
+	res.append(GetString(GetTradeCosts(rs_from, rs_to, tradingPost)));
     }
 
-    if(Resource::GOLD != rs_from && Resource::GOLD != rs_to) str = "1/";
-
-    String::AddInt(str, GetTradeCosts(rs_from, rs_to, tradingPost));
+    return res;
 }
 
 u16 GetTradeCosts(u8 rs_from, u8 rs_to, bool tradingPost)
 {
-    const u8 markets = tradingPost ? 3 : world.GetMyKingdom().GetCountMarketplace();
-    
+    const u8 markets = tradingPost ? 3 : world.GetKingdom(Settings::Get().CurrentColor()).GetCountMarketplace();
+
     if(rs_from == rs_to) return 0;
 
     switch(rs_from)
@@ -763,7 +639,7 @@ u16 GetTradeCosts(u8 rs_from, u8 rs_to, bool tradingPost)
 	case Resource::SULFUR:
 	case Resource::CRYSTAL:
 	case Resource::GEMS:
-    	    
+
     	    switch(rs_to)
     	    {
     		// sale costly
@@ -786,7 +662,7 @@ u16 GetTradeCosts(u8 rs_from, u8 rs_to, bool tradingPost)
         	    else
         	    if(8 <  markets) return SALE_COSTLY9;
         	    break;
-        	
+
 		// change costly to costly
 		case Resource::MERCURY:
 		case Resource::SULFUR:
@@ -837,11 +713,11 @@ u16 GetTradeCosts(u8 rs_from, u8 rs_to, bool tradingPost)
 
 	// gold
 	case Resource::GOLD:
-    	    
+
     	    switch(rs_to)
     	    {
     		default: break;
-        	
+
 		// buy costly
 		case Resource::MERCURY:
 		case Resource::SULFUR:
