@@ -21,6 +21,7 @@
  ***************************************************************************/
 
 #include <algorithm>
+#include <cctype>
 #include "agg.h"
 #include "settings.h"
 #include "text.h"
@@ -190,19 +191,14 @@ void TextAscii::Blit(u16 ax, u16 ay, const u16 maxw, Surface & dst)
             break;
 	}
 
-	dst.Blit(sprite, ax, ay + 2 + oy);
+	sprite.Blit(ax, ay + 2 + oy, dst);
 	ax += sprite.w();
     }
 }
 
 #ifdef WITH_TTF
-TextUnicode::TextUnicode(const std::string & msg, Font::type_t ft) : TextInterface(ft), message(2 * msg.size(), 0)
+TextUnicode::TextUnicode(const std::string & msg, Font::type_t ft) : TextInterface(ft), message(String::UTF8_to_UNICODE(msg))
 {
-    if(msg.size())
-    {
-	String::UTF8_to_UNICODE(&message[0], msg.c_str(), msg.size());
-	while(0 == message.back()) message.pop_back();
-    }
 }
 
 TextUnicode::TextUnicode(const u16 *pt, size_t sz, Font::type_t ft) : TextInterface(ft), message(pt, pt+sz)
@@ -228,13 +224,7 @@ bool TextUnicode::isspace(u16 c)
 
 void TextUnicode::SetText(const std::string & msg)
 {
-    if(msg.size())
-    {
-	message.resize(2 * msg.size(), 0);
-	std::fill(message.begin(), message.end(), 0);
-	String::UTF8_to_UNICODE(&message[0], msg.c_str(), msg.size());
-	while(0 == message.back()) message.pop_back();
-    }
+    message = String::UTF8_to_UNICODE(msg);
 }
 
 void TextUnicode::SetFont(const Font::type_t & ft)
@@ -263,8 +253,8 @@ u8 TextUnicode::CharWidth(u16 c, Font::type_t f)
 u8 TextUnicode::CharHeight(Font::type_t f)
 {
     return Font::SMALL == f || Font::YELLOW_SMALL ?
-        AGG::Cache::Get().GetSmallFont().Height() :
-        AGG::Cache::Get().GetMediumFont().Height();
+        (AGG::Cache::Get().GetSmallFont().Height() + 2) :
+        (AGG::Cache::Get().GetMediumFont().Height() + 8);
 }
 
 u8 TextUnicode::CharAscent(Font::type_t f)
@@ -360,7 +350,7 @@ void TextUnicode::Blit(u16 ax, u16 ay, const u16 maxw, Surface & dst)
 	const Surface & sprite = AGG::GetUnicodeLetter(*it, font);
 	if(!sprite.isValid()) return;
 
-	dst.Blit(sprite, ax, ay);
+	sprite.Blit(ax, ay, dst);
 	ax += sprite.w();
     }
 }
@@ -548,9 +538,7 @@ void TextBox::Set(const std::string & msg, Font::type_t ft, u16 width)
 #ifdef WITH_TTF
     if(Settings::Get().Unicode())
     {
-	std::vector<u16> unicode(2 * msg.size(), 0);
-	String::UTF8_to_UNICODE(&unicode[0], msg.c_str(), msg.size());
-	while(0 == unicode.back()) unicode.pop_back();
+	std::vector<u16> unicode = String::UTF8_to_UNICODE(msg);
 
         const u16 sep = '\n';
         std::vector<u16> substr;
