@@ -124,7 +124,6 @@ const settings_t settingsFHeroes2[] =
     { Settings::GAME_SAVE_REWRITE_CONFIRM,	_("game: always confirm for rewrite savefile"),		},
     { Settings::GAME_ALSO_CONFIRM_AUTOSAVE,	_("game: also confirm autosave"),			},
     { Settings::GAME_REMEMBER_LAST_FOCUS,	_("game: remember last focus"),				},
-    { Settings::GAME_REMEMBER_LAST_FILENAME,	_("game: remember last filename"),			},
     { Settings::GAME_BATTLE_SHOW_GRID,		_("game: battle show grid"),				},
     { Settings::GAME_BATTLE_SHOW_MOUSE_SHADOW,	_("game: battle mouse shadow")				},
     { Settings::GAME_BATTLE_SHOW_MOVE_SHADOW,	_("game: battle move shadow"),				},
@@ -147,6 +146,7 @@ const settings_t settingsFHeroes2[] =
     { Settings::WORLD_ARTSPRING_SEPARATELY_VISIT,_("world: Artesian Springs have two separately visitable squares (h3 ver)"), },
     { Settings::WORLD_STARTHERO_LOSSCOND4HUMANS,_("world: Starting heroes as Loss Conditions for Human Players"), },
     { Settings::WORLD_1HERO_HIRED_EVERY_WEEK,	_("world: Only 1 hero can be hired by the one player every week"), },
+    { Settings::CASTLE_1HERO_HIRED_EVERY_WEEK,	_("world: each castle allows one hero to be recruited every week"), },
     { Settings::WORLD_DWELLING_ACCUMULATE_UNITS,_("world: Outer creature dwellings should accumulate units"), },
     { Settings::WORLD_USE_UNIQUE_ARTIFACTS_ML,	_("world: use unique artifacts for morale/luck"),       },
     { Settings::WORLD_USE_UNIQUE_ARTIFACTS_RS,	_("world: use unique artifacts for resource affecting"),},
@@ -1379,11 +1379,6 @@ bool Settings::ExtGameAutosaveOn(void) const
     return ExtModes(GAME_AUTOSAVE_ON);
 }
 
-bool Settings::ExtGameRememberLastFilename(void) const
-{
-    return ExtModes(GAME_REMEMBER_LAST_FILENAME);
-}
-
 bool Settings::ExtGameUseFade(void) const
 {
     return video_mode.w == 640 && video_mode.h == 480 && ExtModes(GAME_USE_FADE);
@@ -1479,6 +1474,11 @@ bool Settings::ExtWorldOneHeroHiredEveryWeek(void) const
     return ExtModes(WORLD_1HERO_HIRED_EVERY_WEEK);
 }
 
+bool Settings::ExtCastleOneHeroHiredEveryWeek(void) const
+{
+    return ExtModes(CASTLE_1HERO_HIRED_EVERY_WEEK);
+}
+
 bool Settings::ExtWorldDwellingsAccumulateUnits(void) const
 {
     return ExtModes(WORLD_DWELLING_ACCUMULATE_UNITS);
