@@ -142,7 +142,7 @@ public:
 	GAME_EVIL_INTERFACE		= 0x10001000,
 	GAME_HIDE_INTERFACE		= 0x10002000,
 	GAME_ALSO_CONFIRM_AUTOSAVE	= 0x10004000,
-	GAME_REMEMBER_LAST_FILENAME	= 0x10008000,
+	//UNUSED			= 0x10008000,
 	GAME_DYNAMIC_INTERFACE		= 0x10010000,
         GAME_BATTLE_SHOW_GRID		= 0x10020000,
         GAME_BATTLE_SHOW_MOUSE_SHADOW	= 0x10040000,
@@ -199,6 +199,7 @@ public:
 	WORLD_USE_UNIQUE_ARTIFACTS_SS	= 0x30000800,
 	WORLD_DISABLE_BARROW_MOUNDS	= 0x30001000,
 	WORLD_EXT_OBJECTS_CAPTURED 	= 0x30004000,
+	CASTLE_1HERO_HIRED_EVERY_WEEK	= 0x30008000,
 
 	BATTLE_ARCHMAGE_RESIST_BAD_SPELL= 0x40001000,
 	BATTLE_MAGIC_TROOP_RESIST	= 0x40002000,
@@ -324,6 +325,7 @@ public:
     bool ExtCastleAllowFlash(void) const;
     bool ExtCastleGuildRestorePointsTurn(void) const;
     bool ExtCastleAllowRecruitSpecialHeroes(void) const;
+    bool ExtCastleOneHeroHiredEveryWeek(void) const;
     bool ExtBattleShowDamage(void) const;
     bool ExtBattleSoftWait(void) const;
     bool ExtBattleMagicTroopCanResist(void) const;
@@ -336,7 +338,6 @@ public:
     bool ExtBattleShowMouseShadow(void) const;
     bool ExtBattleShowMoveShadow(void) const;
     bool ExtGameRememberLastFocus(void) const;
-    bool ExtGameRememberLastFilename(void) const;
     bool ExtGameContinueAfterVictory(void) const;
     bool ExtGameRewriteConfirm(void) const;
     bool ExtGameAutosaveConfirm(void) const;
